<?php

namespace Drp\AdfsSocialiteProvider\Http\Controllers;

use Drp\AdfsSocialiteProvider\TemporaryUser;
use GuzzleHttp\Exception\ClientException;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Facades\Auth;
use Laravel\Socialite\AbstractUser as SocialiteUser;
use Laravel\Socialite\Facades\Socialite;

class HandleProviderCallback
{
    public function __invoke(): RedirectResponse
    {
        try {
            $user = Socialite::driver('adfs')->user();
        } catch (ClientException $exception) {
            return redirect()->to('/')->withError('Unable to login. Please try again later.');
        }

        $this->logInUser($user);

        return app('adfs.redirect');
    }

    protected function logInUser($user): void
    {
        $guardName = config('adfs.login.guard');
        $driver = config("auth.guards.$guardName.driver");

        if ($driver === 'adfs') {
            $authUser = new TemporaryUser(
                $user->id,
                $user['username'],
                $user->given_name . ' ' . $user->family_name
            );

            $guard = Auth::createSessionDriver($guardName, [
                'provider' => 'users',
            ]);
            $guard->login($authUser, false);

            return;
        }

        $authUser = $this->findOrCreateUser($user);
        auth($guardName)->login($authUser, false);
    }

    /**
     * Return user if exists; create and return if doesn't
     */
    private function findOrCreateUser(SocialiteUser $adfsUser)
    {
        $userModel = config('adfs.auth.user_model');

        $user = $userModel::where('adfs_id', $adfsUser->id)->first();

        if ($user === null) {
            $user = new $userModel([
                'name' => $adfsUser->given_name . ' ' . $adfsUser->family_name,
                'email' => $adfsUser->email,
                'adfs_id' => $adfsUser->id,
            ]);
        }

        if (method_exists($user, 'mapAdfsUserAttributes')) {
            $user->mapAdfsUserAttributes($adfsUser);
        }

        return tap($user)->save();
    }
}
